
{******************************************}
{                                          }
{             FastReport v4.0              }
{         Accuracer enduser components     }
{                                          }

// Copyright (c) AidAim Software
// http://www.aidaim.com
// 17 February 2009
// modified - FastReport 4.9.21:
// 08 February 2010
{                                          }
{******************************************}

unit frxACRComponents;

interface

{$I frx.inc}

{$I frxACRVer.inc}

uses
  Windows, Classes, SysUtils,  frxClass, frxCustomDB, DB,
  ACRMain
{$IFDEF ACR5H} // Accuracer 5.0 and higher
  ,ACRComMain
{$ENDIF}

{$IFDEF Delphi6}
, Variants
{$ENDIF}
{$IFDEF QBUILDER}
 , fqbClass
{$ENDIF};


type
  TfrxACRComponents = class(TfrxDBComponents)
  private
    FDefaultDatabase: TACRDatabase;
    FOldComponents: TfrxACRComponents;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetDescription: String; override;
  published
    property DefaultDatabase: TACRDatabase read FDefaultDatabase write FDefaultDatabase;
  end;

  TfrxACRDatabase = class(TfrxCustomDatabase)
  private
    FDatabase: TACRDatabase;
  protected
    procedure SetConnected(Value: Boolean); override;
    procedure SetDatabaseName(const Value: String); override;
    procedure SetLoginPrompt(Value: Boolean); override;
    procedure SetParams(Value: TStrings); override;
    procedure SetFileName(Value: string);    // TS
    procedure SetLocalDatabase(Value: Boolean); // TS
    procedure SetConnectionParams(Value: TACRConnectionParamsEditor);  // TS
    procedure SetCryptoParams(Value: TACRCryptoParamsEditor); // TS
    procedure SetKeepConnection(Value: boolean); // TS
    procedure SetInMemory(Value: boolean); // TS
    function GetConnected: Boolean; override;
    function GetDatabaseName: String; override;   // TS
    function GetFileName: string;    // TS
    function GetLoginPrompt: Boolean; override;
    function GetParams: TStrings; override;
    function GetConnectionParams: TACRConnectionParamsEditor;     // TS
    function GetCryptoParams: TACRCryptoParamsEditor; // TS
    function GetLocalDatabase: boolean; // TS
    function GetExclusive: boolean; // TS
    function GetInMemory: boolean; // TS
    function GetKeepConnection: boolean; // TS
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function GetDescription: String; override;
    procedure SetLogin(const Login, Password: String); override;
    property Database: TACRDatabase read FDatabase write fDatabase;
  published
    property DatabaseName: String read GetDatabaseName write SetDatabaseName;
    property DatabaseFilename: string read GetFilename write SetFileName;    // TS
    property ConnectionParams: TACRConnectionParamsEditor read GetConnectionParams write SetConnectionParams;
    property CryptoParams    : TACRCryptoParamsEditor     read GetCryptoParams     write SetCryptoParams;
 //   property Params;     // TS
    property LocalDatabase: boolean read GetLocalDatabase write SetLocalDatabase;     // TS
    property InMemory: boolean read GetInMemory write SetInMemory;                    // TS
    property KeepConnection: boolean read GetKeepConnection write SetKeepConnection;  // TS
    property Connected;
  end;

  TfrxACRTable = class(TfrxCustomTable)
  private
    FTable: TACRTable;
    procedure SetDatabase(const Value: string);
    function GetDatabase: string;
  protected
  //  procedure Notification(AComponent: TComponent; Operation: TOperation); override;    // TS
    procedure SetMaster(const Value: TDataSource); override;
    procedure SetMasterFields(const Value: String); override;
    procedure SetIndexFieldNames(const Value: String); override;
    procedure SetIndexName(const Value: String); override;
    procedure SetTableName(const Value: String); override;
    function GetIndexFieldNames: String; override;
    function GetIndexName: String; override;
    function GetTableName: String; override;
  public
    constructor Create(AOwner: TComponent); override;
    constructor DesignCreate(AOwner: TComponent; Flags: Word); override;
    class function GetDescription: String; override;
  //  procedure BeforeStartReport; override;             // TS marked out
    property Table: TACRTable read FTable;
  published
    property DatabaseName: String read GetDatabase write SetDatabase;
  end;

  TfrxACRQuery = class(TfrxCustomQuery)
  private
    FQuery: TACRQuery;
{$IFDEF ACR5H}
    FStrings: TStringList;
{$ENDIF}
  //  FLock: Boolean;
    procedure SetDatabase(Value: String);
    function GetDatabase: string;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure OnChangeSQL(Sender: TObject); override;
    procedure SetMaster(const Value: TDataSource); override;
    procedure SetSQL(Value: TStrings); override;
    function GetSQL: TStrings; override;

  public
    constructor Create(AOwner: TComponent); override;
    constructor DesignCreate(AOwner: TComponent; Flags: Word); override;
    destructor Destroy; override;
    class function GetDescription: String; override;
    procedure BeforeStartReport; override;
    procedure UpdateParams; override;
{$IFDEF QBUILDER}
    function QBEngine: TfqbEngine; override;
{$ENDIF}
    property Query: TACRQuery read FQuery;
  published
    property DatabaseName: string read GetDatabase write SetDatabase;
  end;

{$IFDEF QBUILDER}
  TfrxACREngine = class(TfqbEngine)
  private
    FQuery: TACRQuery;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ReadTableList(ATableList: TStrings); override;
    procedure ReadFieldList(const ATableName: string; var AFieldList: TfqbFieldList); override;
    function ResultDataSet: TDataSet; override;
    procedure SetSQL(const Value: string); override;
  published
    property Query: TACRQuery read FQuery write FQuery;
  end;
{$ENDIF}


var
  ACRComponents: TfrxACRComponents;

implementation

{$R *.res}

uses 
  frxACRRTTI, 
{$IFNDEF NO_EDITORS}
  frxACREditor, 
{$ENDIF}
  frxDsgnIntf, frxRes;


{ frxParamsToTParameters }

procedure frxParamsToTParameters(Query: TfrxCustomQuery; Params: TParams);
var
  i: Integer;
  Item: TfrxParamItem;
begin
  for i := 0 to Params.Count - 1 do
    if Query.Params.IndexOf(Params[i].Name) <> -1 then
    begin
      Item := Query.Params[Query.Params.IndexOf(Params[i].Name)];
      Params[i].DataType := Item.DataType;
      if Trim(Item.Expression) <> '' then
        if not (Query.IsLoading or Query.IsDesigning) then
        begin
          Query.Report.CurObject := Query.Name;
          Item.Value := Query.Report.Calc(Item.Expression);
        end;
      Params[i].Value := Item.Value;
    end;
end;

{ TfrxACRComponents }

constructor TfrxACRComponents.Create(AOwner: TComponent);
begin
  inherited;
  FOldComponents := ACRComponents;
  ACRComponents := Self;
end;

destructor TfrxACRComponents.Destroy;
begin
  if ACRComponents = Self then
    ACRComponents := FOldComponents;
  inherited;
end;

function TfrxACRComponents.GetDescription: String;
begin
  Result := 'ACR';
end;


{ TfrxACRDatabase }

constructor TfrxACRDatabase.Create(AOwner: TComponent);
begin
  inherited;
  FDatabase := TACRDatabase.Create(nil);
  Component := FDatabase;
end;

destructor TfrxACRDatabase.Destroy;
begin
  inherited;
end;

class function TfrxACRDatabase.GetDescription: String;
begin
  Result := 'ACR Database';
end;

function TfrxACRDatabase.GetExclusive: boolean;
begin
  result := fDatabase.Exclusive;
end;

function TfrxACRDatabase.GetFileName: string;
begin
  Result := fDatabase.DatabaseFileName;     // TS
end;

function TfrxACRDatabase.GetInMemory: boolean;
begin
  result := fDatabase.InMemory;
end;

function TfrxACRDatabase.GetKeepConnection: boolean;
begin
  result := fDatabase.KeepConnection;
end;

function TfrxACRDatabase.GetConnected: Boolean;
begin
  Result := FDatabase.Connected;
end;

function TfrxACRDatabase.GetConnectionParams: TACRConnectionParamsEditor;
begin
  if assigned(fDatabase.ConnectionParams) then
    Result := fDatabase.ConnectionParams   // TS
  else
    Result := nil;
end;

function TfrxACRDatabase.GetCryptoParams: TACRCryptoParamsEditor;
begin
  Result := fDatabase.CryptoParams;
end;

function TfrxACRDatabase.GetDatabaseName: String;
begin
  Result := FDatabase.DatabaseName;
end;

function TfrxACRDatabase.GetLocalDatabase: boolean;
begin
 result := Fdatabase.LocalDatabase;
end;

function TfrxACRDatabase.GetLoginPrompt: Boolean;
begin
  Result := False;
end;

function TfrxACRDatabase.GetParams: TStrings;
begin
  Result := nil;
end;

procedure TfrxACRDatabase.SetConnected(Value: Boolean);
begin
  BeforeConnect(Value);
  FDatabase.Connected := Value;
end;

procedure TfrxACRDatabase.SetConnectionParams(
  Value: TACRConnectionParamsEditor);
begin
  FDatabase.ConnectionParams.Assign(Value); // TS
end;

procedure TfrxACRDatabase.SetCryptoParams(Value: TACRCryptoParamsEditor);
begin
  fDatabase.CryptoParams.Assign(Value); // TS
end;

procedure TfrxACRDatabase.SetDatabaseName(const Value: String);
begin
  FDatabase.DatabaseName := Value;
end;

procedure TfrxACRDatabase.SetFileName(Value: string);
begin
  inherited;
  fDatabase.DatabaseFileName := Value;
end;

procedure TfrxACRDatabase.SetInMemory(Value: boolean);
begin
  fDatabase.InMemory := value;
end;

procedure TfrxACRDatabase.SetKeepConnection(Value: boolean);
begin
  fDatabase.KeepConnection := value;
end;

procedure TfrxACRDatabase.SetLoginPrompt(Value: Boolean);
begin
//  FDatabase.LoginPrompt := Value;
end;

procedure TfrxACRDatabase.SetParams(Value: TStrings);
begin
//  FDatabase.Params := Value;
end;

procedure TfrxACRDatabase.SetLocalDatabase(Value: Boolean);
begin
 fDatabase.LocalDatabase := value;
end;

procedure TfrxACRDatabase.SetLogin(const Login, Password: String);
begin
// this method is used by "New connection" wizard
// for example (IBX):
// Params.Text := 'user_name=' + Login + #13#10 + 'password=' + Password;
end;


{ TfrxACRTable }

constructor TfrxACRTable.Create(AOwner: TComponent);
begin
  FTable := TACRTable.Create(nil);
  DataSet := FTable;
  SetDatabase('');
  inherited;
end;

constructor TfrxACRTable.DesignCreate(AOwner: TComponent; Flags: Word);
var
  i: Integer;
  l: TList;
begin
  inherited;
  l := Report.AllObjects;
  for i := 0 to l.Count - 1 do
    if TObject(l[i]) is TfrxACRDatabase then
    begin
      SetDatabase(TfrxACRDatabase(l[i]).databaseName);
      break;
    end;

end;

function TfrxACRTable.GetDatabase: string;
begin
  Result := fTable.Databasename;
end;

class function TfrxACRTable.GetDescription: String;
begin
  Result := 'ACR Table';
end;

//procedure TfrxACRTable.Notification(AComponent: TComponent; Operation: TOperation);       // TS marked out.
//begin
//  inherited;
//  if (Operation = opRemove) and (AComponent = FDatabase) then
//    SetDatabase('');
// end;

procedure TfrxACRTable.SetDatabase(const Value: string);
begin
  if Value <> '' then
    FTable.DatabaseName := Value
  else if (ACRComponents <> nil) and (ACRComponents.DefaultDatabase.DatabaseName <> '') then
    FTable.DatabaseName := ACRComponents.DefaultDatabase.DatabaseName
  else
    FTable.DatabaseName := '';
end;

function TfrxACRTable.GetIndexFieldNames: String;
begin
  Result := FTable.IndexFieldNames;
end;

function TfrxACRTable.GetIndexName: String;
begin
  Result := FTable.IndexName;
end;

function TfrxACRTable.GetTableName: String;
begin
  Result := FTable.TableName;
end;

procedure TfrxACRTable.SetIndexFieldNames(const Value: String);
begin
  FTable.IndexFieldNames := Value;
end;

procedure TfrxACRTable.SetIndexName(const Value: String);
begin
  FTable.IndexName := Value;
end;

procedure TfrxACRTable.SetTableName(const Value: String);
begin
  FTable.TableName := Value;
end;

procedure TfrxACRTable.SetMaster(const Value: TDataSource);
begin
  FTable.MasterSource := Value;
end;

procedure TfrxACRTable.SetMasterFields(const Value: String);
begin
  FTable.MasterFields := Value;
end;

{procedure TfrxACRTable.BeforeStartReport;
begin
  SetDatabase(fTable.Database);
end; }


{ TfrxACRQuery }

constructor TfrxACRQuery.Create(AOwner: TComponent);
begin
  FQuery := TACRQuery.Create(nil);
{$IFDEF ACR5H}
  FStrings := TStringList.Create;
{$ENDIF}
  SQL := TSTringlist.Create;
  Dataset := FQuery;
  SetDatabase('');
  inherited;
end;

constructor TfrxACRQuery.DesignCreate(AOwner: TComponent; Flags: Word);
var
  i: Integer;
  l: TList;
begin
  inherited;
    l := Report.AllObjects;
    for i := 0 to l.Count - 1 do
      if TObject(l[i]) is TfrxACRDatabase then
      begin
        SetDatabase(TfrxACRDatabase(l[i]).DatabaseName);
        break;
    end;
end;

destructor TfrxACRQuery.Destroy;
begin
{$IFDEF ACR5H}
  FStrings.Free;
{$ENDIF}
  inherited;
end;

function TfrxACRQuery.GetDatabase: string;
begin
 Result := fQuery.Databasename;
end;

class function TfrxACRQuery.GetDescription: String;
begin
  Result := frxResources.Get('ACR Query');
end;

procedure TfrxACRQuery.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
end;

function TfrxACRQuery.GetSQL: TStrings;
begin
{$IFDEF ACR5H}
  FQuery.SQL.ExportToTstrings(FStrings);
  Result := FStrings;
{$ELSE}
  Result := FQuery.SQL;
{$ENDIF}
end;

procedure TfrxACRQuery.SetSQL(Value: TStrings);
begin
  FQuery.SQL.Assign(Value);
end;

procedure TfrxACRQuery.SetDatabase(Value: string);
begin

  if Value = '' then
  begin
    if (ACRComponents <> nil) and (ACRComponents.DefaultDatabase <> nil) then
      fQuery.DatabaseName := ACRComponents.DefaultDatabase.DatabaseName
    else
      fQuery.Databasename := '';
  end
  else
     FQuery.Databasename := Value;
end;

procedure TfrxACRQuery.SetMaster(const Value: TDataSource);
begin
  FQuery.DataSource := Value;
end;

procedure TfrxACRQuery.UpdateParams;
begin
  frxParamsToTParameters(Self, FQuery.Params);
end;

procedure TfrxACRQuery.OnChangeSQL(Sender: TObject);
var
  i, ind: Integer;
  Param: TfrxParamItem;
  QParam: TParam;
begin


  inherited;

  { fill datatype automatically, if possible }
  for i := 0 to FQuery.Params.Count - 1 do
  begin
    QParam := FQuery.Params[i];
    ind := Params.IndexOf(QParam.Name);
    if ind <> -1 then
    begin
      Param := Params[ind];
      if (Param.DataType = ftUnknown) and (QParam.DataType <> ftUnknown) then
        Param.DataType := QParam.DataType;
    end;
  end;
end;

procedure TfrxACRQuery.BeforeStartReport;
begin
  { needed to update parameters }
  SQL.Text := SQL.Text;
end;


{$IFDEF QBUILDER}
function TfrxACRQuery.QBEngine: TfqbEngine;
begin
  Result := TfrxACREngine.Create(nil);
  TFrxACREngine(Result).FQuery := Self.FQuery;
  TfrxACREngine(Result).FQuery.DatabaseName := FQuery.DatabaseName;
end;
{$ENDIF}


{$IFDEF QBUILDER}
constructor TfrxACREngine.Create(AOwner: TComponent);
begin
  inherited;
  FQuery := nil;;
end;

destructor TfrxACREngine.Destroy;
begin
  inherited;
end;

procedure TfrxACREngine.ReadFieldList(const ATableName: string;
  var AFieldList: TfqbFieldList);
var
  TempTable: TACRTable;
  Fields: TFieldDefs;
  i: Integer;
  tmpField: TfqbField;
begin
  AFieldList.Clear;
  if (FQuery = nil) then Exit;
  TempTable := TACRTable.Create(Self);
  TempTable.DatabaseName := FQuery.DatabaseName;
  TempTable.TableName := ATableName;
  Fields := TempTable.FieldDefs;
  try
    try
      TempTable.Active := True;
      tmpField:= TfqbField(AFieldList.Add);
      tmpField.FieldName := '*';
      for i := 0 to Fields.Count - 1 do
      begin
        tmpField := TfqbField(AFieldList.Add);
        tmpField.FieldName := Fields.Items[i].Name;
        tmpField.FieldType := Ord(Fields.Items[i].DataType)
      end;
    except
    end;
  finally
    TempTable.Free;
  end;
end;

procedure TfrxACREngine.ReadTableList(ATableList: TStrings);
var
  tmpTable: TACRTable;
begin
  ATableList.Clear;
  if (FQuery <> nil) then
   if (FQuery.DatabaseName <> '') then
   begin
    tmpTable := TACRTable.Create(nil);
    tmpTable.DatabaseName := fQuery.DatabaseName;
    tmpTable.GetTableNames(ATableList);
    tmpTable.Free;
   end;
end;

function TfrxACREngine.ResultDataSet: TDataSet;
begin
  Result := FQuery;
end;

procedure TfrxACREngine.SetSQL(const Value: string);
begin
  if (FQuery <> nil) then
   FQuery.SQL.Text := Value;
end;
{$ENDIF}

{
// v.3 - earlier v.4 of Fast Report
initialization
  frxObjects.RegisterObject1(TfrxACRDataBase, nil, 'Accuracer Database', 'ACR', 0, 37);
  frxObjects.RegisterObject1(TfrxACRTable, nil, 'Accuracer Table', 'ACR', 0, 38);
  frxObjects.RegisterObject1(TfrxACRQuery, nil, 'Accuracer Query', 'ACR', 0, 39);
}


initialization
  frxObjects.RegisterObject1(TfrxACRDataBase, nil, '', {$IFDEF DB_CAT}'DATABASES'{$ELSE}''{$ENDIF}, 0, 37);
  frxObjects.RegisterObject1(TfrxACRTable, nil, '', {$IFDEF DB_CAT}'TABLES'{$ELSE}''{$ENDIF}, 0, 38);
  frxObjects.RegisterObject1(TfrxACRQuery, nil, '', {$IFDEF DB_CAT}'QUERIES'{$ELSE}''{$ENDIF}, 0, 39);

finalization
//  CatBmp.Free;
  frxObjects.UnRegister(TfrxACRDataBase);
  frxObjects.UnRegister(TfrxACRTable);
  frxObjects.UnRegister(TfrxACRQuery);


end.