
{******************************************}
{                                          }
{             FastReport v4.0              }
{ Accuracer components design editors      }
{                                          }

// Copyright
{                                          }
{******************************************}

unit frxACREditor;

interface

{$I frx.inc}

{$I frxACRVer.inc}

implementation

uses
  Windows, Classes, SysUtils, Forms, Dialogs, frxACRComponents, frxCustomDB,
  frxDsgnIntf, frxRes
  ,ACRMain
{$IFDEF ACR5H} // Accuracer 5.0 and higher
  ,ACRComMain
{$ENDIF}
  , frxClass
{$IFDEF Delphi6}
, Variants
{$ENDIF}
;


type
  TfrxACRDatabaseFileNameProperty = class(TfrxStringProperty)
  public
    function GetAttributes: TfrxPropertyAttributes; override;
    function Edit: Boolean; override;
  end;

  TfrxACRConnectionProperty = class(TfrxClassProperty)      // TS
  public
    function GetValue: String; override;
    procedure SetValue(const Value: string); override;
    function GetAttributes: TfrxPropertyAttributes; override;
  end;

  TFrxACRCryptoProperty = class(TfrxClassProperty) // TS
  public
    function GetValue: String; override;
    procedure SetValue(const Value: string); override;
    function GetAttributes: TfrxPropertyAttributes; override;
  end;

  TfrxACRDatabaseNameProperty = class(TfrxStringProperty)
  public
    function GetAttributes: TfrxPropertyAttributes; override;
    procedure GetValues; override;
    procedure SetValue(const Value: string); override;
  end;

  TfrxACRTableNameProperty = class(TfrxStringProperty)
  public
    function GetAttributes: TfrxPropertyAttributes; override;
    procedure GetValues; override;
    procedure SetValue(const Value: String); override;
  end;

  TfrxACRIndexNameProperty = class(TfrxStringProperty)
  public
    function GetAttributes: TfrxPropertyAttributes; override;
    procedure GetValues; override;
  end;


{ TfrxDatabaseNameProperty }

function TfrxACRDatabaseFileNameProperty.GetAttributes: TfrxPropertyAttributes;
begin
  Result := [paDialog];
end;

function TfrxACRDatabaseFileNameProperty.Edit: Boolean;
var
  SaveConnected: Boolean;
begin
 with TOpenDialog.Create(nil) do
  begin
    InitialDir := GetCurrentDir;
    Filter := 'Select a database';      // TS
    Result := Execute;
    if Result then
      with TfrxACRDatabase(Component).Database do
      begin
        SaveConnected := Connected;
        Connected := False;
       // DatabaseName := FileName;     // TS
        TfrxACRDatabase(Component).DatabaseFilename := Filename;        // TS
        Connected := SaveConnected;
      end;
    Free;
  end;
end;


{ TfrxDatabaseProperty }

function TfrxACRDatabaseNameProperty.GetAttributes: TfrxPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paSortList];
end;

procedure TfrxACRDatabaseNameProperty.GetValues;
var

  i: Integer;
  l: TList;
begin
 
  if (ACRComponents <> nil) and (ACRComponents.DefaultDatabase <> nil) then
      Values.add(ACRComponents.DefaultDatabase.DatabaseName);




  l := TfrxACRTable(Component).Report.AllObjects;
  for i := 0 to l.Count - 1 do
    if TObject(l[i]) is TfrxACRDatabase then
    begin
      values.Add(TfrxACRDatabase(l[i]).databaseName)
    end;
end;


procedure TfrxACRDatabaseNameProperty.SetValue(const Value: string);
begin
  inherited;
  SetStrValue(value);
end;

{ TfrxTableNameProperty }

function TfrxACRTableNameProperty.GetAttributes: TfrxPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paSortList];
end;

procedure TfrxACRTableNameProperty.GetValues;
begin
  inherited;
  with TfrxACRTable(Component).Table do
  //  if Database <> nil then
  //    DataBase.GetTablesList(Values);
     GetTableNames(Values);
end;

procedure TfrxACRTableNameProperty.SetValue(const Value: String);
begin
  inherited;
  Designer.UpdateDataTree;
end;


{ TfrxIndexProperty }

function TfrxACRIndexNameProperty.GetAttributes: TfrxPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList];
end;

procedure TfrxACRIndexNameProperty.GetValues;
var
  i: Integer;
begin
  inherited;
  try
    with TfrxACRTable(Component).Table do
      if (TableName <> '') and (IndexDefs <> nil) then
      begin
        IndexDefs.Update;
        for i := 0 to IndexDefs.Count - 1 do
          if IndexDefs[i].Name <> '' then
            Values.Add(IndexDefs[i].Name);
      end;
  except
  end;
end;


{ TfrxACRConnectionProperty }


function TfrxACRConnectionProperty.GetAttributes: TfrxPropertyAttributes;
begin
    Result := [paMultiSelect, paDialog, paSubProperties, paReadOnly];
end;

function TfrxACRConnectionProperty.GetValue: String;
begin
  result := GetStrValue;
end;

procedure TfrxACRConnectionProperty.SetValue(const Value: string);
begin
  inherited;
  SetStrValue(value);
end;

{ TFrxACRCryptoProperty }

function TFrxACRCryptoProperty.GetAttributes: TfrxPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paSubProperties, paReadOnly];
end;

function TFrxACRCryptoProperty.GetValue: String;
begin
  result := GetStrValue;
end;

procedure TFrxACRCryptoProperty.SetValue(const Value: string);
begin
  inherited;
  SetStrValue(value);
end;

initialization
  frxPropertyEditors.Register(TypeInfo(String), TfrxACRDataBase, 'DatabaseFileName',
    TfrxACRDataBaseFileNameProperty);
  frxPropertyEditors.Register(TypeInfo(TACRConnectionParamsEditor), TfrxACRDatabase, 'ConnectionParams', TfrxACRConnectionProperty);
  frxPropertyEditors.Register(TypeInfo(TACRCryptoParamsEditor), TfrxACRDatabase, 'CryptoParams', TfrxACRCryptoProperty);
  frxPropertyEditors.Register(TypeInfo(string), TfrxACRTable, 'DatabaseName',
    TfrxACRDatabaseNameProperty);
  frxPropertyEditors.Register(TypeInfo(string), TfrxACRQuery, 'DatabaseName',
    TfrxACRDatabaseNameProperty);
  frxPropertyEditors.Register(TypeInfo(String), TfrxACRTable, 'TableName',
    TfrxACRTableNameProperty);
  frxPropertyEditors.Register(TypeInfo(String), TfrxACRTable, 'IndexName',
    TfrxACRIndexNameProperty);

end.
